@@ -12,6 +12,7 @@ use Path::Tiny;
 use Pod::POM;
 use v5.32;
 use Text::Wrap;
+use Config::INI::Reader;
 use Config::Model 2.150 qw/cme/;
 use Pod::POM::View::Text;
 use JSON;
@@ -22,7 +23,7 @@ $Text::Wrap::columns = 72;
 use feature qw/signatures/;
 no warnings qw/experimental::signatures/;
 
-our @EXPORT_OK = qw/setup_debian_files setup_git commit set_remote/;
+our @EXPORT_OK = qw/setup_debian_files setup_git commit set_remote upgrade/;
 
 sub add_missing_tag {
     my $version = get_version_from_meta6();
@@ -34,15 +35,23 @@ sub add_missing_tag {
     return $tag;
 }
 
-sub setup_git ($arg) {
+sub setup_git ($config, $arg) {
     my $init = 0;
     my $cwd = path('.')->absolute;
+    my $upstream_remote = $config->{upstream_remote};
+    my $upstream_branch = $config->{upstream_branch};
+    my $debian_branch = $config->{debian_branch};
 
     my ($pkg_dir, $git, $package, $tag);
     if ($cwd->child('.git')->is_dir) {
+        my $repo = Git->repository();
+        if (grep {$_ eq 'upstream'} $repo->command('remote')) {
+            say "renaming ambiguous remote name from upstream to $upstream_remote";
+            $repo->command_oneline(remote => rename => upstream => $upstream_remote);
+        }
+
         $package = $cwd->basename;
-        $git = `git remote get-url upstream`;
-        chomp $git;
+        $git = $repo->command_oneline(remote => 'get-url', $upstream_remote);
         $pkg_dir = $cwd;
         $tag = $arg->{tag};
     }
@@ -56,17 +65,17 @@ sub setup_git ($arg) {
             die "missing 'git' arg\n" unless $git;
             $pkg_dir = $cwd->child($package);
             if (not $pkg_dir->exists) {
-                Git::command_oneline(qw/clone --origin upstream/, $git, $package);
+                Git::command_oneline(qw/clone --origin/, $upstream_remote, $git, $package);
                 chdir $pkg_dir;
-                my @refs = split m!/!, Git::command_oneline(qw(symbolic-ref refs/remotes/upstream/HEAD));
+                my @refs = split m!/!, Git::command_oneline('symbolic-ref', "refs/remotes/$upstream_remote/HEAD");
                 my $branch = pop @refs;
-                Git::command_oneline('branch', '--move', $branch, 'upstream');
+                Git::command_oneline('branch', '--move', $branch, $upstream_branch);
                 if ($tag !~ /\d/) {
                     # need to get actual version and tag
                     Git::command_oneline(checkout => $tag);
                     $tag = add_missing_tag();
                 }
-                system("git checkout -b debian/sid $tag");
+                Git::command_oneline('checkout', '-b', $debian_branch, $tag);
                 $init = 1;
             }
             else {
@@ -79,17 +88,17 @@ sub setup_git ($arg) {
 
 sub set_remote ($name, $url) {
     my @remotes = Git::command_oneline(qw/remote show/);
-    if (grep {$_ eq 'origin'} @remotes) {
-        my $current = Git::command_oneline(qw/remote get-url origin/);
+    if (grep {$_ eq $name} @remotes) {
+        my $current = Git::command_oneline(qw/remote get-url/, $name);
         if ($current ne $url) {
-            say "Setting remote origin to $url...";
-            Git::command_oneline(qw/remote set-url origin/, $url);
+            say "Setting remote $name to $url...";
+            Git::command_oneline(qw/remote set-url/, $name, $url);
         }
     }
     else {
-        say "Creating remote origin to $url...";
-        Git::command_oneline(qw/remote add origin/, $url);
-        Git::command_oneline(qw/remote set-url origin --push/, $url);
+        say "Creating remote $name to $url...";
+        Git::command_oneline(qw/remote add/, $name, $url);
+        Git::command_oneline(qw/remote set-url/, $name, '--push', $url);
     }
 }
 
@@ -150,7 +159,12 @@ sub setup_dependencies($control, $package, $pkg_dir) {
     return unless $data->{depends};
 
     foreach my $raku_dep ($data->{depends}->@*) {
+        # skip when dependency has hints, don't know how to use them
+        # see https://design.raku.org/S22.html#depends
+        next if ref $raku_dep;
+
         next if is_core_dep($raku_dep);
+
         my $dep = raku_dep_to_debian_dep($raku_dep);
         say "check dependency $dep";
         $control->load(qq{source Build-Depends:.ensure("$dep")}.
@@ -187,10 +201,6 @@ control source
  Depends="\${misc:Depends}","\${raku:Depends}"
 EOL
 
-    if ($pkg_dir->child('t')->is_dir) {
-        $root->load('control source Build-Depends:.ensure(prove6)');
-    }
-
     my $desc = get_description($pkg_dir);
     my $desc_obj = $root->grab("control binary:$package Description");
     if ($desc or not $desc_obj->fetch(check => 'no')) {
@@ -416,7 +426,7 @@ sub install_docs($root, $package, $pkg_dir) {
     }
 }
 
-sub setup_debian_files ($pkg_dir, $git, $package, $tag, $init) {
+sub setup_debian_files ($config, $pkg_dir, $git, $package, $tag, $init) {
     my $patch_name = setup_patches ($pkg_dir);
 
     my $instance = cme(application => 'dpkg', name => $package);
@@ -463,12 +473,27 @@ sub setup_debian_files ($pkg_dir, $git, $package, $tag, $init) {
 pristine-tar = False
 pristine-tar-commit = False
 upstream-tag = $tag_pattern
-debian-branch = debian/sid
+debian-branch = $config->{debian_branch}
+upstream-branch = $config->{upstream_branch}
 !);
     }
 
     return $root;
 }
 
+sub gbp_conf ($arg, $class = 'DEFAULT') {
+    my $config_hash = Config::INI::Reader->read_file('debian/gbp.conf');
+    return $config_hash->{$class}{$arg};
+}
+
+sub upgrade ($config, $version) {
+    my $tag = gbp_conf('upstream-tag') eq 'v%(version)s' ? "v$version" : $version;
+
+    Git::command_oneline('fetch', '--all');
+    Git::command_oneline('checkout', $config->{upstream_branch});
+    Git::command_oneline('merge', $tag);
+    Git::command_oneline('checkout', $config->{debian_branch});
+    Git::command_oneline('merge', $config->{upstream_branch});
+}
 
 1;
